@@ -10,7 +10,6 @@ a :class:`ModelMock`) via which the user can instantiate model instances
 """
 from collections import defaultdict
 import itertools
-import sys
 
 import networkx
 
@@ -58,7 +57,7 @@ class World:
     that this increases the memory consumption and simulation time.
 
     """
-    def __init__(self, sim_config, mosaik_config=None, execution_graph=False):
+    def __init__(self, sim_config, mosaik_config=None, debug=False):
         self.sim_config = sim_config
         """The config dictionary that tells mosaik how to start a simulator."""
 
@@ -77,17 +76,19 @@ class World:
         """Mosaik's server socket."""
 
         self.df_graph = networkx.DiGraph()
-        """The directed dataflow graph for this scenario."""
+        """The directed data-flow :func:`graph <networkx.DiGraph>` for this
+        scenario."""
 
         self.entity_graph = networkx.Graph()
-        """The graph of related entities. Nodes are ``(sid, eid)`` tuples.
-        Each note has an attribute *entity* with an :class:`Entity`."""
+        """The :func:`graph <networkx.Graph>` of related entities. Nodes are
+        ``(sid, eid)`` tuples.  Each note has an attribute *entity* with an
+        :class:`Entity`."""
 
         self.sim_progress = 0
         """Progress of the current simulation."""
 
         self._debug = False
-        if execution_graph:
+        if debug:
             self._debug = True
             self.execution_graph = networkx.DiGraph()
 
@@ -97,7 +98,8 @@ class World:
         # _df_outattr[sim_id][entity_id] = [attr_1, attr2, ...]
         self._df_outattr = defaultdict(lambda: defaultdict(list))
         # Cache for simulation results
-        self._df_cache = util.OrderedDefaultdict(dict)
+        self._df_cache = defaultdict(dict)
+        self._df_cache_min_time = 0
 
     def start(self, sim_name, **sim_params):
         """Start the simulator named *sim_name* and return a
@@ -115,14 +117,14 @@ class World:
     def connect(self, src, dest, *attr_pairs, async_requests=False):
         """Connect the *src* entity to *dest* entity.
 
-        Establish a dataflow for each ``(src_attr, dest_attr)`` tuple in
+        Establish a data-flow for each ``(src_attr, dest_attr)`` tuple in
         *attr_pairs*. If *src_attr* and *dest_attr* have the same name, you
         you can optionally only pass one of them as a single string.
 
         Raise a :exc:`~mosaik.exceptions.ScenarioError` if both entities share
         the same simulator instance, if at least one (src. or dest.) attribute
         in *attr_pairs* does not exist, or if the connection would introduce
-        a cycle in the dataflow (e.g., A → B → C → A).
+        a cycle in the data-flow (e.g., A → B → C → A).
 
         If the *dest* simulator may make asyncronous requests to mosaik to
         query data from *src* (or set data to it), *async_requests* should be
@@ -141,7 +143,7 @@ class World:
             raise ScenarioError('At least one attribute does not exist: %s' %
                                 ', '.join('%s.%s' % x for x in missing_attrs))
 
-        # Add edge and check for cycles and the dataflow graph.
+        # Add edge and check for cycles and the data-flow graph.
         self.df_graph.add_edge(src.sid, dest.sid,
                                async_requests=async_requests)
         if not networkx.is_directed_acyclic_graph(self.df_graph):
@@ -183,7 +185,7 @@ class World:
             outputs_by_sim[entity.sid][entity.eid] = attributes
 
         def request_data():
-            requests = {self.sims[sid].get_data(outputs): sid
+            requests = {self.sims[sid].proxy.get_data(outputs): sid
                         for sid, outputs in outputs_by_sim.items()}
             try:
                 results = yield self.env.all_of(requests)
@@ -212,24 +214,36 @@ class World:
 
         return results
 
-    def run(self, until):
+    def run(self, until, rt_factor=None, rt_strict=False):
         """Start the simulation until the simulation time *until* is reached.
 
-        Return the current simulation time (>= *until*).
+        In order to perform real-time simulations, you can set *rt_factor* to
+        a number > 0. An rt-factor of 1 means that 1 simulation time unit
+        (usually a second) takes 1 second in real-time. An rt-factor 0f 0.5
+        will let the simulation run twice as fast as real-time.
 
-        This method should only be called once!
+        If the simulators are too slow for the rt-factor you chose, mosaik
+        prints by default only a warning. In order to raise
+        a :exc:`RuntimeError`, you can set *rt_strict* to ``True``.
+
+        Before this method returns, it stops all simulators and closes mosaik's
+        server socket. So this method should only be called once.
 
         """
+        if self.srv_sock is None:
+            raise RuntimeError('Simulation has already been run and can only '
+                               'be run once for a World instance.')
+
         print('Starting simulation.')
         if self._debug:
             import mosaik._debug as dbg
             dbg.enable()
         try:
-            util.sync_process(scheduler.run(self, until), self)
+            util.sync_process(scheduler.run(self, until, rt_factor, rt_strict),
+                              self)
             print('Simulation finished successfully.')
         except KeyboardInterrupt:
             print('Simulation canceled. Terminating ...')
-            sys.exit(1)
         finally:
             self.shutdown()
             if self._debug:
@@ -240,6 +254,7 @@ class World:
         for sim in self.sims.values():
             util.sync_process(sim.stop(), self, ignore_errors=True)
         self.srv_sock.close()
+        self.srv_sock = None
 
     def _check_attributes(self, src, dest, attr_pairs):
         """Check if *src* and *dest* have the attributes in *attr_pairs*.
@@ -251,7 +266,7 @@ class World:
         """
         entities = [src, dest]
         emeta = [e.sim.meta['models'][e.type] for e in entities]
-        any_inputs = [False, emeta[1].get('any_inputs', False)]
+        any_inputs = [False, emeta[1]['any_inputs']]
         attr_errors = []
         for attr_pair in attr_pairs:
             for i, attr in enumerate(attr_pair):
@@ -276,19 +291,27 @@ class ModelFactory:
         self._world = world
         self._env = world.env
         self._sim = sim
-        self._model_cache = {}
+
+        # Create a ModelMock for every public model
+        for model, props in self.meta['models'].items():
+            if props['public']:
+                setattr(self, model, ModelMock(self._world, model, self._sim))
+
+        # Bind extra_methods to this instance:
+        for meth in self.meta['extra_methods']:
+            def wrapper(*args, **kwargs):
+                return util.sync_call(sim, meth, args, kwargs)
+            wrapper.__name__ = meth
+            setattr(self, meth, wrapper)
 
     def __getattr__(self, name):
-        if name not in self.meta['models']:
-            raise ScenarioError('Model factory for "%s" has no model "%s".' %
-                                (self._sim.sid, name))
-        if not self.meta['models'][name]['public']:
-            raise ScenarioError('Model "%s" is not public.' % name)
-
-        if name not in self._model_cache:
-            self._model_cache[name] = ModelMock(self._world, name, self._sim)
-
-        return self._model_cache[name]
+        # Implemented in order to improve error messages.
+        models = self.meta['models']
+        if name in models and not models[name]['public']:
+            raise AttributeError('Model "%s" is not public.' % name)
+        else:
+            raise AttributeError('Model factory for "%s" has no model and no '
+                                 'function "%s".' % (self._sim.sid, name))
 
 
 class ModelMock:
@@ -325,19 +348,8 @@ class ModelMock:
         """
         self._check_params(**model_params)
 
-        # We have to start a SimPy process to make the "create()" call
-        # behave like it was synchronous.
-        def create_proc():
-            try:
-                entities = yield self._sim.create(num, self._name,
-                                                  **model_params)
-                return entities
-            except ConnectionError as e:
-                msg = ('Simulator "%s" closed its connection while creating %s'
-                       ' instances of "%s".' % (self._sim_id, num, self._name))
-                raise SimulationError(msg, e) from None
-
-        entities = util.sync_process(create_proc(), self._world)
+        entities = util.sync_call(self._sim, 'create', [num, self._name],
+                                  model_params)
         assert len(entities) == num, (
             '%d entities were requested but %d were created.' %
             (num, len(entities)))
@@ -371,7 +383,7 @@ class ModelMock:
 
             entity_set.append(entity)
             entity_graph.add_node(entity.full_id, sim=sim_name, type=e['type'])
-            for rel in e['rel']:
+            for rel in e.get('rel', []):
                 entity_graph.add_edge(entity.full_id, FULL_ID % (sim_id, rel))
 
         return entity_set
