@@ -242,23 +242,45 @@ def valid_api_version(simulator_version, expected_version):
 
 
 class SimProxy:
-    """Simple proxy/facade for in-process simulators."""
-    def __init__(self, name, sid, meta):
+    """Handler for an external simulator.
+
+    It stores its simulation state and own the proxy object to the external
+    simulator.
+
+    """
+    def __init__(self, name, sid, meta, world):
         self.name = name
         self.sid = sid
         self.meta = meta
+        self._world = world
+
+        # Meta data and remote method checks
+        api_methods = [
+            # "init" was called before the SimProxy was created
+            'create',
+            'step',
+            'get_data',
+        ]
+        # Set default value for optional "extra_methods" property
+        extra_methods = meta.setdefault('extra_methods', [])
+
+        self._check_model_and_meth_names(meta['models'], api_methods,
+                                         extra_methods)
+
+        # Set default value for optional "any_inputs" property
+        for model, props in meta['models'].items():
+            props.setdefault('any_inputs', False)
+
+        # Actual proxy object
+        self.proxy = self._get_proxy(api_methods + extra_methods)
+
+        # Simulation state
         self.last_step = float('-inf')
         self.next_step = 0
         self.input_buffer = {}  # Buffer used by "MosaikRemote.set_data()"
         self.sim_proc = None  # SimPy process
         self.step_required = None  # SimPy event
 
-        # Bind proxy calls to API methods to this instance. "init()" is missing
-        # here, because it is called before the SimProxy is created
-        remote_methods = ['create', 'step', 'get_data']
-        for name in remote_methods:
-            setattr(self, name, self._proxy_call(name))
-
     def stop(self):
         """Stop the simulator behind the proxy.
 
@@ -267,15 +289,33 @@ class SimProxy:
         """
         raise NotImplementedError
 
-    def _proxy_call(self, name):
+    def _get_proxy(self, methods):
         raise NotImplementedError
 
+    def _check_model_and_meth_names(self, models, api_methods, extra_methods):
+        """Check if there are any overlaps in model names and reserved API
+        methods as well as in them and extra API methods.
+
+        Raise a :exc:`~mosaik.exception.ScenarioError` if that's the case.
+
+        """
+        models = list(models)
+        illegal_models = set(models) & set(api_methods)
+        if illegal_models:
+            raise ScenarioError('Simulator "%s" uses illegal model names: %s' %
+                                (self.sid, ', '.join(illegal_models)))
+
+        illegal_meths = set(models + api_methods) & set(extra_methods)
+        if illegal_meths:
+            raise ScenarioError('Simulator "%s" uses illegal extra method '
+                                'names: %s' %
+                                (self.sid, ', '.join(illegal_meths)))
+
 
 class LocalProcess(SimProxy):
     """Proxy for internal simulators."""
     def __init__(self, name, sid, meta, inst, world):
         self._inst = inst
-        self._env = world.env
 
         # Add MosaikRemote and patch its RPC methods to return events:
         inst.mosaik = MosaikRemote(world, sid)
@@ -288,20 +328,22 @@ class LocalProcess(SimProxy):
                 setattr(inst.mosaik, attr,
                         mosaik_api.get_wrapper(func, world.env))
 
-        super().__init__(name, sid, meta)
+        super().__init__(name, sid, meta, world)
 
     def stop(self):
         """Yield a triggered event but do nothing else."""
-        yield self._env.event().succeed()
+        self._inst.finalize()
+        yield self._world.env.event().succeed()
 
-    def _proxy_call(self, name):
-        """Return a wrapper method for the simulators method *name*.
-
-        The wrapper method will return an event with the original return value
-        as its value.
-
-        """
-        return mosaik_api.get_wrapper(getattr(self._inst, name), self._env)
+    def _get_proxy(self, methods):
+        """Return a proxy for the local simulator."""
+        proxy_dict = {
+            name: mosaik_api.get_wrapper(getattr(self._inst, name),
+                                         self._world.env)
+            for name in methods
+        }
+        Proxy = type('Proxy', (), proxy_dict)
+        return Proxy
 
 
 class RemoteProcess(SimProxy):
@@ -309,11 +351,10 @@ class RemoteProcess(SimProxy):
     def __init__(self, name, sid, meta, proc, rpc_con, world):
         self._proc = proc
         self._rpc_con = rpc_con
-        self._env = world.env
         self._mosaik_remote = MosaikRemote(world, sid)
         self._stop_timeout = world.config['stop_timeout']
         rpc_con.router = self._mosaik_remote.rpc
-        super().__init__(name, sid, meta)
+        super().__init__(name, sid, meta, world)
 
     def stop(self):
         """Send a *stop* message to the process represented by this proxy and
@@ -322,7 +363,7 @@ class RemoteProcess(SimProxy):
         """
         try:
             yield (self._rpc_con.remote.stop() |
-                   self._env.timeout(self._stop_timeout))
+                   self._world.env.timeout(self._stop_timeout))
             print('Simulator "%s" did not close its connection in time.' %
                   self.sid)
             self._rpc_con.close()
@@ -332,9 +373,9 @@ class RemoteProcess(SimProxy):
         if self._proc:
             self._proc.wait()
 
-    def _proxy_call(self, name):
-        """Return the method *name* of the remote process."""
-        return getattr(self._rpc_con.remote, name)
+    def _get_proxy(self, methods):
+        """Return a proxy object for the remote simulator."""
+        return self._rpc_con.remote
 
 
 class MosaikRemote:
@@ -430,10 +471,15 @@ class MosaikRemote:
         data = {}
         missing = collections.defaultdict(
             lambda: collections.defaultdict(list))
+        dfg = self.world.df_graph
+        dest_sid = self.sim_id
         # Try to get data from cache
         for full_id, attr_names in attrs.items():
-            data[full_id] = {}
             sid, eid = full_id.split(FULL_ID_SEP, 1)
+            # Check if async_requests are enabled.
+            self._assert_async_requests(dfg, sid, dest_sid)
+
+            data[full_id] = {}
             for attr in attr_names:
                 try:
                     data[full_id][attr] = cache_slice[sid][eid][attr]
@@ -445,7 +491,7 @@ class MosaikRemote:
             dep = self.world.sims[sid]
             assert (dep.next_step > sim.last_step and
                     dep.last_step <= sim.last_step)
-            dep_data = yield dep.get_data(attrs)
+            dep_data = yield dep.proxy.get_data(attrs)
             for eid, vals in dep_data.items():
                 # Maybe there's already an entry for full_id, so we need
                 # to update the dict in that case.
@@ -463,9 +509,31 @@ class MosaikRemote:
 
         """
         sims = self.world.sims
+        dfg = self.world.df_graph
+        dest_sid = self.sim_id
         for src_full_id, dest in data.items():
             for full_id, attributes in dest.items():
                 sid, eid = full_id.split(FULL_ID_SEP, 1)
+                self._assert_async_requests(dfg, sid, dest_sid)
                 inputs = sims[sid].input_buffer.setdefault(eid, {})
                 for attr, val in attributes.items():
                     inputs.setdefault(attr, {})[src_full_id] = val
+
+    def _assert_async_requests(self, dfg, src_sid, dest_sid):
+        """Check if async. requests are allowed from *dest_sid* to *src_sid*
+        and raise a :exc:`ScenarioError` if not."""
+        data = {
+            'src': src_sid,
+            'dest': dest_sid,
+        }
+        if dest_sid not in dfg[src_sid]:
+            raise ScenarioError(
+                'No connection from "%(src)s" to "%(dest)s": You need to '
+                'connect entities from both simulators and set '
+                '"async_requests=True".' % data)
+        if dfg[src_sid][dest_sid]['async_requests'] is not True:
+            raise ScenarioError(
+                'Async. requests not enabled for the connection from '
+                '"%(src)s" to "%(dest)s". Add the argument '
+                '"async_requests=True" to the connection of entities from '
+                '"%(src)s" to "%(dest)s".' % data)
