@@ -2,7 +2,6 @@
 This module contains some utility functions and classes.
 
 """
-import collections
 import random
 import sys
 
@@ -11,26 +10,6 @@ from simpy.io.network import RemoteException
 from mosaik.exceptions import SimulationError
 
 
-class OrderedDefaultdict(collections.OrderedDict):
-    """Mixes :class:`~collections.OrderedDict` with
-    :class:`~collections.defaultdict`."""
-    def __init__(self, *args, **kwargs):
-        if not args:
-            self.default_factory = None
-        else:
-            if not (args[0] is None or hasattr(args[0], '__call__')):
-                raise TypeError('first argument must be callable')
-            self.default_factory = args[0]
-            args = args[1:]
-        super().__init__(*args, **kwargs)
-
-    def __missing__(self, key):
-        if self.default_factory is None:
-            raise KeyError(key)
-        self[key] = default = self.default_factory()
-        return default
-
-
 def sync_process(generator, world, *, ignore_errors=False):
     """Synchronously execute a SimPy process defined by the generator object
     *generator*.
@@ -62,6 +41,41 @@ def sync_process(generator, world, *, ignore_errors=False):
         sys.exit(1)
 
 
+def sync_call(sim, funcname, args, kwargs):
+    """Start a SimPy process to make the *func()* call to a simulator behave
+    like it was synchronous.
+
+    Return the result of the *func()* call.
+
+    Raise an :exc:`~mosaik.exceptions.SimulationError` if an exception occurs.
+
+    """
+    # We have to start a SimPy process to make the "create()" call
+    # behave like it was synchronous.
+    def proc():
+        try:
+            func = getattr(sim.proxy, funcname)
+            ret = yield func(*args, **kwargs)
+            return ret
+        except ConnectionError as e:
+            err_msg = ('Simulator "%s" closed its connection while executing '
+                       '%s(*%s, **%s)' % (sim.sid, funcname, args, kwargs))
+            raise SimulationError(err_msg, e) from None
+
+    return sync_process(proc(), sim._world)
+
+
+def connect_many_to_one(world, src_set, dest, *attrs, async_requests=False):
+    """:meth:`~mosaik.scenario.World.connect` each entity in *src_set*
+    to *dest*.
+
+    See the :meth:`~mosaik.scenario.World.connect` for more details.
+
+    """
+    for src in src_set:
+        world.connect(src, dest, *attrs, async_requests=async_requests)
+
+
 def connect_randomly(world, src_set, dest_set, *attrs, evenly=True,
                      max_connects=float('inf')):
     """Randomly :meth:`~mosaik.scenario.World.connect` the entities from
