@@ -3,14 +3,17 @@ import sys
 from example_sim.mosaik import ExampleSim
 from simpy.io.json import JSON as JsonRpc
 from simpy.io.message import Message
+from simpy.io.network import RemoteException
 from simpy.io.packet import PacketUTF8 as Packet
+import mosaik_api
 import pytest
 
 from mosaik import scenario
 from mosaik import simmanager
 from mosaik.exceptions import ScenarioError
 import mosaik
-import mosaik_api
+
+from .util import SimMock
 
 
 sim_config = {
@@ -29,6 +32,9 @@ sim_config = {
     'Fail': {
         'cmd': 'python -c "import time; time.sleep(0.2)"',
     },
+    'SimMock': {
+        'python': 'tests.util:SimMock',
+    },
 }
 
 
@@ -36,7 +42,8 @@ sim_config = {
 def world():
     world = scenario.World(sim_config)
     yield world
-    world.shutdown()
+    if world.srv_sock:
+        world.shutdown()
 
 
 def test_start(world, monkeypatch):
@@ -250,16 +257,31 @@ def test_valid_api_version(version, valid):
 
 def test_sim_proxy():
     """SimProxy should not be instantiateable."""
-    pytest.raises(NotImplementedError, simmanager.SimProxy, 'spam', 'id', {})
+    pytest.raises(NotImplementedError, simmanager.SimProxy, 'spam', 'id',
+                  {'models': {}}, None)
+
+
+def test_sim_proxy_illegal_model_names(world):
+    pytest.raises(ScenarioError, simmanager.LocalProcess, '', 0,
+                  {'models': {'step': {}}}, SimMock(), world)
+
+
+def test_sim_proxy_illegal_extra_methods(world):
+    pytest.raises(ScenarioError, simmanager.LocalProcess, '', 0,
+                  {'models': {'A': {}}, 'extra_methods': ['step']}, SimMock(),
+                  world)
+    pytest.raises(ScenarioError, simmanager.LocalProcess, '', 0,
+                  {'models': {'A': {}}, 'extra_methods': ['A']}, SimMock(),
+                  world)
 
 
 def test_sim_proxy_stop_impl():
     class Test(simmanager.SimProxy):
         # Does not implement SimProxy.stop(). Should raise an error.
-        def _proxy_call(self, name):
+        def _get_proxy(self, name):
             return None
 
-    t = Test('spam', 'id', {})
+    t = Test('spam', 'id', {'models': {}}, None)
     pytest.raises(NotImplementedError, t.stop)
 
 
@@ -279,6 +301,14 @@ def test_local_process():
     assert sp.step_required is None
 
 
+def test_local_process_finalized(world):
+    """Test that ``finalize()`` is called for local processes (issue #23)."""
+    sm = world.start('SimMock')
+    assert sm._sim._inst.finalized is False
+    world.run(until=1)
+    assert sm._sim._inst.finalized is True
+
+
 def _rpc_get_progress(mosaik, world):
     """Helper for :func:`test_mosaik_remote()` that checks the "get_progress()"
     RPC."""
@@ -352,19 +382,46 @@ def _rpc_set_data(mosaik, world):
     }
 
 
-@pytest.mark.parametrize('rpc', [
-    _rpc_get_progress,
-    _rpc_get_related_entities,
-    _rpc_get_data,
-    _rpc_set_data,
+def _rpc_get_data_err1(mosaik, world):
+    """Required simulator not connected to us."""
+    yield mosaik.get_data({'Z.2': []})
+
+
+def _rpc_get_data_err2(mosaik, world):
+    """Async-requests flag not set for connection."""
+    yield mosaik.get_data({'Y.2': []})
+
+
+def _rpc_set_data_err1(mosaik, world):
+    """Required simulator not connected to us."""
+    yield mosaik.set_data({'src': {'Z.2': {'val': 42}}})
+
+
+def _rpc_set_data_err2(mosaik, world):
+    """Async-requests flag not set for connection."""
+    yield mosaik.set_data({'src': {'Y.2': {'val': 42}}})
+
+
+@pytest.mark.parametrize(('rpc', 'err'), [
+    (_rpc_get_progress, None),
+    (_rpc_get_related_entities, None),
+    (_rpc_get_data, None),
+    (_rpc_set_data, None),
+    (_rpc_get_data_err1, ScenarioError),
+    (_rpc_get_data_err2, ScenarioError),
+    (_rpc_set_data_err1, RemoteException),
+    (_rpc_set_data_err2, RemoteException),
 ])
-def test_mosaik_remote(rpc):
+def test_mosaik_remote(rpc, err):
     backend = simmanager.backend
     world = scenario.World({})
     env = world.env
 
     edges = [(0, 1), (0, 2), (1, 2), (2, 3)]
     edges = [('X.%s' % x, 'X.%s' % y) for x, y in edges]
+    world.df_graph.add_edge('X', 'X', async_requests=True)
+    world.df_graph.add_edge('Y', 'X', async_requests=False)
+    world.df_graph.add_node('Z')
     world.entity_graph.add_edges_from(edges)
     for node in world.entity_graph:
         world.entity_graph.add_node(node, sim='ExampleSim', type='A')
@@ -388,10 +445,14 @@ def test_mosaik_remote(rpc):
     def greeter():
         sock = yield world.srv_sock.accept()
         rpc_con = JsonRpc(Packet(sock))
-        proxy = simmanager.RemoteProcess('X', 'X', {}, None, rpc_con, world)
+        proxy = simmanager.RemoteProcess('X', 'X', {'models': {}}, None,
+                                         rpc_con, world)
         proxy.last_step = proxy.next_step = 1
         world.sims['X'] = proxy
 
     env.process(greeter())
-    env.run(env.process(simulator()))
+    if err is None:
+        env.run(env.process(simulator()))
+    else:
+        pytest.raises(err, env.run, env.process(simulator()))
     world.srv_sock.close()
